#include <cassert>

#include <gp-bnb/partition.hpp>

partition::partition(graph* g) : supergraph_(g) {
    // Assigns all vertices to none
    for (unsigned int i = 0; i < g->num_vertices(); i++) {
        vertex_assignments_[i] = none;
    }

    // Initializes vertex counting map
    vertices_[sg_a] = 0;
    vertices_[sg_b] = 0;
    vertices_[none] = g->num_vertices();
}

unsigned int partition::num_vertices_of(subgraph sg) const {
    return vertices_.at(sg);
}

unsigned int partition::current_objective() const {
    return current_objective_;
}

partition::subgraph partition::assigned_subgraph_of(vertex_id v) const {
    return vertex_assignments_[v-1];
}

void partition::assign_vertex(vertex_id v, subgraph sg) {
    assert(vertex_assignments_[v-1] != none);
    
    // Increments current objectives
    for (auto const& target : supergraph_->get_adjacency(v)) {
        if (vertex_assignments_[target-1] == -sg) {
            current_objective_++;    
        }
    }
    
    // Assigns vertex to subgraph
    vertex_assignments_[v-1] = sg;
    vertices_[sg]++;
    vertices_[none]--;
}

void partition::unassign_vertex(vertex_id v) {
    subgraph sg = vertex_assignments_[v-1];
    
    assert(sg == none);
    
    // Decrements current objectives
    for (auto const& target : supergraph_->get_adjacency(v)) {
        if (vertex_assignments_[target-1] == -sg) {
            current_objective_--;
        }
    }
    
    // Reverts assignment to subgraph 
    vertex_assignments_[v-1] = none;
    vertices_[sg]--;
    vertices_[none]++;
}
